# SPDX-FileCopyrightText: © 2025 Tenstorrent AI ULC
#
# SPDX-License-Identifier: Apache-2.0
import torch
import torch_ttnn
import pytest
import pickle
import ttnn
from pathlib import Path
from tests.utils import calculate_accuracy, render_metric_string_list_to_input_args_kwargs


class AtenModule(torch.nn.Module):
    def __init__(self):
        super().__init__()

    def forward(self, *args, **kwargs):
        return torch.ops.aten._native_batch_norm_legit_no_training.default(*args, **kwargs)[0]


metrics = []


def save_pickle(obj, base_path, filename):
    p = Path(base_path)
    p.mkdir(parents=True, exist_ok=True)
    pickle_out_path = p / f"{filename}.pickle"
    with open(pickle_out_path, "wb") as f:
        pickle.dump(obj, f)


def teardown_module(module):
    print(metrics)
    save_pickle(metrics, "metrics-autogen-op/ALL", "aten._native_batch_norm_legit_no_training.default")


@pytest.mark.parametrize(
    "input_strings",
    [
        [
            "Tensor<[1, 64, 30, 40]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 30, 40]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 60, 80]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 60, 80]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 120, 160]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 120, 160]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 14, 56, 56]> input = ?",
            "Optional[Tensor]<[14]> weight = ?",
            "Optional[Tensor]<[14]> bias = ?",
            "Tensor<[14]> running_mean = ?",
            "Tensor<[14]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 40, 56, 56]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 68, 56, 56]> input = ?",
            "Optional[Tensor]<[68]> weight = ?",
            "Optional[Tensor]<[68]> bias = ?",
            "Tensor<[68]> running_mean = ?",
            "Tensor<[68]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 56, 56]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 16, 28, 28]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 28, 28, 28]> input = ?",
            "Optional[Tensor]<[28]> weight = ?",
            "Optional[Tensor]<[28]> bias = ?",
            "Tensor<[28]> running_mean = ?",
            "Tensor<[28]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 46, 28, 28]> input = ?",
            "Optional[Tensor]<[46]> weight = ?",
            "Optional[Tensor]<[46]> bias = ?",
            "Tensor<[46]> running_mean = ?",
            "Tensor<[46]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 78, 28, 28]> input = ?",
            "Optional[Tensor]<[78]> weight = ?",
            "Optional[Tensor]<[78]> bias = ?",
            "Tensor<[78]> running_mean = ?",
            "Tensor<[78]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 134, 28, 28]> input = ?",
            "Optional[Tensor]<[134]> weight = ?",
            "Optional[Tensor]<[134]> bias = ?",
            "Tensor<[134]> running_mean = ?",
            "Tensor<[134]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 28, 28]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 20, 28, 28]> input = ?",
            "Optional[Tensor]<[20]> weight = ?",
            "Optional[Tensor]<[20]> bias = ?",
            "Tensor<[20]> running_mean = ?",
            "Tensor<[20]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 34, 28, 28]> input = ?",
            "Optional[Tensor]<[34]> weight = ?",
            "Optional[Tensor]<[34]> bias = ?",
            "Tensor<[34]> running_mean = ?",
            "Tensor<[34]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 58, 28, 28]> input = ?",
            "Optional[Tensor]<[58]> weight = ?",
            "Optional[Tensor]<[58]> bias = ?",
            "Tensor<[58]> running_mean = ?",
            "Tensor<[58]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 98, 28, 28]> input = ?",
            "Optional[Tensor]<[98]> weight = ?",
            "Optional[Tensor]<[98]> bias = ?",
            "Tensor<[98]> running_mean = ?",
            "Tensor<[98]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 168, 28, 28]> input = ?",
            "Optional[Tensor]<[168]> weight = ?",
            "Optional[Tensor]<[168]> bias = ?",
            "Tensor<[168]> running_mean = ?",
            "Tensor<[168]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 320, 28, 28]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 68, 14, 14]> input = ?",
            "Optional[Tensor]<[68]> weight = ?",
            "Optional[Tensor]<[68]> bias = ?",
            "Tensor<[68]> running_mean = ?",
            "Tensor<[68]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 116, 14, 14]> input = ?",
            "Optional[Tensor]<[116]> weight = ?",
            "Optional[Tensor]<[116]> bias = ?",
            "Tensor<[116]> running_mean = ?",
            "Tensor<[116]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 196, 14, 14]> input = ?",
            "Optional[Tensor]<[196]> weight = ?",
            "Optional[Tensor]<[196]> bias = ?",
            "Tensor<[196]> running_mean = ?",
            "Tensor<[196]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 334, 14, 14]> input = ?",
            "Optional[Tensor]<[334]> weight = ?",
            "Optional[Tensor]<[334]> bias = ?",
            "Tensor<[334]> running_mean = ?",
            "Tensor<[334]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 640, 14, 14]> input = ?",
            "Optional[Tensor]<[640]> weight = ?",
            "Optional[Tensor]<[640]> bias = ?",
            "Tensor<[640]> running_mean = ?",
            "Tensor<[640]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 272, 7, 7]> input = ?",
            "Optional[Tensor]<[272]> weight = ?",
            "Optional[Tensor]<[272]> bias = ?",
            "Tensor<[272]> running_mean = ?",
            "Tensor<[272]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 462, 7, 7]> input = ?",
            "Optional[Tensor]<[462]> weight = ?",
            "Optional[Tensor]<[462]> bias = ?",
            "Tensor<[462]> running_mean = ?",
            "Tensor<[462]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1024, 7, 7]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 16, 160, 160]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 160, 160]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 80, 80]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 80, 80]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 72, 80, 80]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 72, 40, 40]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 40, 40, 40]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 120, 40, 40]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 40, 40]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 20, 20]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 80, 20, 20]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 200, 20, 20]> input = ?",
            "Optional[Tensor]<[200]> weight = ?",
            "Optional[Tensor]<[200]> bias = ?",
            "Tensor<[200]> running_mean = ?",
            "Tensor<[200]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 184, 20, 20]> input = ?",
            "Optional[Tensor]<[184]> weight = ?",
            "Optional[Tensor]<[184]> bias = ?",
            "Tensor<[184]> running_mean = ?",
            "Tensor<[184]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 480, 20, 20]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 112, 20, 20]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 20, 20]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 10, 10]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 80, 10, 10]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 480, 10, 10]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 10, 10]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 5, 5]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 512, 5, 5]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 5, 5]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 3, 3]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 3, 3]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 2, 2]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 2, 2]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 2, 2]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 1, 1]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 1, 1]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 96, 112, 112]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 96, 56, 56]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 144, 56, 56]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 144, 28, 28]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 28, 28]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 192, 28, 28]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 192, 14, 14]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 14, 14]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 384, 14, 14]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 96, 14, 14]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 576, 14, 14]> input = ?",
            "Optional[Tensor]<[576]> weight = ?",
            "Optional[Tensor]<[576]> bias = ?",
            "Tensor<[576]> running_mean = ?",
            "Tensor<[576]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 576, 7, 7]> input = ?",
            "Optional[Tensor]<[576]> weight = ?",
            "Optional[Tensor]<[576]> bias = ?",
            "Tensor<[576]> running_mean = ?",
            "Tensor<[576]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 960, 7, 7]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> bias = ?",
            "Tensor<[960]> running_mean = ?",
            "Tensor<[960]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 320, 7, 7]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1280, 7, 7]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 28, 28]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 7, 7]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 56, 56]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1024, 14, 14]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 14, 14]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2048, 7, 7]> input = ?",
            "Optional[Tensor]<[2048]> weight = ?",
            "Optional[Tensor]<[2048]> bias = ?",
            "Tensor<[2048]> running_mean = ?",
            "Tensor<[2048]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 128, 128]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 256, 256]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 128, 128]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 64, 64]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 32, 32]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 16, 16]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 224, 224]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 112, 112]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 512, 512]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 256, 256]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 256, 256]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 128, 128]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 128, 128]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 64, 64]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 64, 64]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 32, 32]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 32, 32]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1024, 16, 16]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 16, 16]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 16, 16]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 32, 32]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 256, 256]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.03",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 160, 56, 56]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 192, 56, 56]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 224, 56, 56]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 160, 28, 28]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 224, 28, 28]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 288, 28, 28]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 352, 28, 28]> input = ?",
            "Optional[Tensor]<[352]> weight = ?",
            "Optional[Tensor]<[352]> bias = ?",
            "Tensor<[352]> running_mean = ?",
            "Tensor<[352]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 384, 28, 28]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 416, 28, 28]> input = ?",
            "Optional[Tensor]<[416]> weight = ?",
            "Optional[Tensor]<[416]> bias = ?",
            "Tensor<[416]> running_mean = ?",
            "Tensor<[416]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 448, 28, 28]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> bias = ?",
            "Tensor<[448]> running_mean = ?",
            "Tensor<[448]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 480, 28, 28]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 14, 14]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 288, 14, 14]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 320, 14, 14]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 352, 14, 14]> input = ?",
            "Optional[Tensor]<[352]> weight = ?",
            "Optional[Tensor]<[352]> bias = ?",
            "Tensor<[352]> running_mean = ?",
            "Tensor<[352]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 416, 14, 14]> input = ?",
            "Optional[Tensor]<[416]> weight = ?",
            "Optional[Tensor]<[416]> bias = ?",
            "Tensor<[416]> running_mean = ?",
            "Tensor<[416]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 448, 14, 14]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> bias = ?",
            "Tensor<[448]> running_mean = ?",
            "Tensor<[448]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 544, 14, 14]> input = ?",
            "Optional[Tensor]<[544]> weight = ?",
            "Optional[Tensor]<[544]> bias = ?",
            "Tensor<[544]> running_mean = ?",
            "Tensor<[544]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 608, 14, 14]> input = ?",
            "Optional[Tensor]<[608]> weight = ?",
            "Optional[Tensor]<[608]> bias = ?",
            "Tensor<[608]> running_mean = ?",
            "Tensor<[608]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 704, 14, 14]> input = ?",
            "Optional[Tensor]<[704]> weight = ?",
            "Optional[Tensor]<[704]> bias = ?",
            "Tensor<[704]> running_mean = ?",
            "Tensor<[704]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 736, 14, 14]> input = ?",
            "Optional[Tensor]<[736]> weight = ?",
            "Optional[Tensor]<[736]> bias = ?",
            "Tensor<[736]> running_mean = ?",
            "Tensor<[736]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 768, 14, 14]> input = ?",
            "Optional[Tensor]<[768]> weight = ?",
            "Optional[Tensor]<[768]> bias = ?",
            "Tensor<[768]> running_mean = ?",
            "Tensor<[768]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 800, 14, 14]> input = ?",
            "Optional[Tensor]<[800]> weight = ?",
            "Optional[Tensor]<[800]> bias = ?",
            "Tensor<[800]> running_mean = ?",
            "Tensor<[800]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 832, 14, 14]> input = ?",
            "Optional[Tensor]<[832]> weight = ?",
            "Optional[Tensor]<[832]> bias = ?",
            "Tensor<[832]> running_mean = ?",
            "Tensor<[832]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 864, 14, 14]> input = ?",
            "Optional[Tensor]<[864]> weight = ?",
            "Optional[Tensor]<[864]> bias = ?",
            "Tensor<[864]> running_mean = ?",
            "Tensor<[864]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 896, 14, 14]> input = ?",
            "Optional[Tensor]<[896]> weight = ?",
            "Optional[Tensor]<[896]> bias = ?",
            "Tensor<[896]> running_mean = ?",
            "Tensor<[896]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 928, 14, 14]> input = ?",
            "Optional[Tensor]<[928]> weight = ?",
            "Optional[Tensor]<[928]> bias = ?",
            "Tensor<[928]> running_mean = ?",
            "Tensor<[928]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 960, 14, 14]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> bias = ?",
            "Tensor<[960]> running_mean = ?",
            "Tensor<[960]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 992, 14, 14]> input = ?",
            "Optional[Tensor]<[992]> weight = ?",
            "Optional[Tensor]<[992]> bias = ?",
            "Tensor<[992]> running_mean = ?",
            "Tensor<[992]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 7, 7]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 544, 7, 7]> input = ?",
            "Optional[Tensor]<[544]> weight = ?",
            "Optional[Tensor]<[544]> bias = ?",
            "Tensor<[544]> running_mean = ?",
            "Tensor<[544]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 608, 7, 7]> input = ?",
            "Optional[Tensor]<[608]> weight = ?",
            "Optional[Tensor]<[608]> bias = ?",
            "Tensor<[608]> running_mean = ?",
            "Tensor<[608]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 640, 7, 7]> input = ?",
            "Optional[Tensor]<[640]> weight = ?",
            "Optional[Tensor]<[640]> bias = ?",
            "Tensor<[640]> running_mean = ?",
            "Tensor<[640]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 704, 7, 7]> input = ?",
            "Optional[Tensor]<[704]> weight = ?",
            "Optional[Tensor]<[704]> bias = ?",
            "Tensor<[704]> running_mean = ?",
            "Tensor<[704]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 736, 7, 7]> input = ?",
            "Optional[Tensor]<[736]> weight = ?",
            "Optional[Tensor]<[736]> bias = ?",
            "Tensor<[736]> running_mean = ?",
            "Tensor<[736]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 768, 7, 7]> input = ?",
            "Optional[Tensor]<[768]> weight = ?",
            "Optional[Tensor]<[768]> bias = ?",
            "Tensor<[768]> running_mean = ?",
            "Tensor<[768]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 800, 7, 7]> input = ?",
            "Optional[Tensor]<[800]> weight = ?",
            "Optional[Tensor]<[800]> bias = ?",
            "Tensor<[800]> running_mean = ?",
            "Tensor<[800]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 832, 7, 7]> input = ?",
            "Optional[Tensor]<[832]> weight = ?",
            "Optional[Tensor]<[832]> bias = ?",
            "Tensor<[832]> running_mean = ?",
            "Tensor<[832]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 864, 7, 7]> input = ?",
            "Optional[Tensor]<[864]> weight = ?",
            "Optional[Tensor]<[864]> bias = ?",
            "Tensor<[864]> running_mean = ?",
            "Tensor<[864]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 896, 7, 7]> input = ?",
            "Optional[Tensor]<[896]> weight = ?",
            "Optional[Tensor]<[896]> bias = ?",
            "Tensor<[896]> running_mean = ?",
            "Tensor<[896]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 928, 7, 7]> input = ?",
            "Optional[Tensor]<[928]> weight = ?",
            "Optional[Tensor]<[928]> bias = ?",
            "Tensor<[928]> running_mean = ?",
            "Tensor<[928]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 992, 7, 7]> input = ?",
            "Optional[Tensor]<[992]> weight = ?",
            "Optional[Tensor]<[992]> bias = ?",
            "Tensor<[992]> running_mean = ?",
            "Tensor<[992]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 240, 56, 56]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 288, 56, 56]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 336, 56, 56]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> bias = ?",
            "Tensor<[336]> running_mean = ?",
            "Tensor<[336]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 384, 56, 56]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 336, 28, 28]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> bias = ?",
            "Tensor<[336]> running_mean = ?",
            "Tensor<[336]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 432, 28, 28]> input = ?",
            "Optional[Tensor]<[432]> weight = ?",
            "Optional[Tensor]<[432]> bias = ?",
            "Tensor<[432]> running_mean = ?",
            "Tensor<[432]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 528, 28, 28]> input = ?",
            "Optional[Tensor]<[528]> weight = ?",
            "Optional[Tensor]<[528]> bias = ?",
            "Tensor<[528]> running_mean = ?",
            "Tensor<[528]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 576, 28, 28]> input = ?",
            "Optional[Tensor]<[576]> weight = ?",
            "Optional[Tensor]<[576]> bias = ?",
            "Tensor<[576]> running_mean = ?",
            "Tensor<[576]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 624, 28, 28]> input = ?",
            "Optional[Tensor]<[624]> weight = ?",
            "Optional[Tensor]<[624]> bias = ?",
            "Tensor<[624]> running_mean = ?",
            "Tensor<[624]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 672, 28, 28]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 720, 28, 28]> input = ?",
            "Optional[Tensor]<[720]> weight = ?",
            "Optional[Tensor]<[720]> bias = ?",
            "Tensor<[720]> running_mean = ?",
            "Tensor<[720]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 768, 28, 28]> input = ?",
            "Optional[Tensor]<[768]> weight = ?",
            "Optional[Tensor]<[768]> bias = ?",
            "Tensor<[768]> running_mean = ?",
            "Tensor<[768]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 432, 14, 14]> input = ?",
            "Optional[Tensor]<[432]> weight = ?",
            "Optional[Tensor]<[432]> bias = ?",
            "Tensor<[432]> running_mean = ?",
            "Tensor<[432]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 528, 14, 14]> input = ?",
            "Optional[Tensor]<[528]> weight = ?",
            "Optional[Tensor]<[528]> bias = ?",
            "Tensor<[528]> running_mean = ?",
            "Tensor<[528]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 624, 14, 14]> input = ?",
            "Optional[Tensor]<[624]> weight = ?",
            "Optional[Tensor]<[624]> bias = ?",
            "Tensor<[624]> running_mean = ?",
            "Tensor<[624]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 720, 14, 14]> input = ?",
            "Optional[Tensor]<[720]> weight = ?",
            "Optional[Tensor]<[720]> bias = ?",
            "Tensor<[720]> running_mean = ?",
            "Tensor<[720]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 816, 14, 14]> input = ?",
            "Optional[Tensor]<[816]> weight = ?",
            "Optional[Tensor]<[816]> bias = ?",
            "Tensor<[816]> running_mean = ?",
            "Tensor<[816]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 912, 14, 14]> input = ?",
            "Optional[Tensor]<[912]> weight = ?",
            "Optional[Tensor]<[912]> bias = ?",
            "Tensor<[912]> running_mean = ?",
            "Tensor<[912]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1008, 14, 14]> input = ?",
            "Optional[Tensor]<[1008]> weight = ?",
            "Optional[Tensor]<[1008]> bias = ?",
            "Tensor<[1008]> running_mean = ?",
            "Tensor<[1008]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1056, 14, 14]> input = ?",
            "Optional[Tensor]<[1056]> weight = ?",
            "Optional[Tensor]<[1056]> bias = ?",
            "Tensor<[1056]> running_mean = ?",
            "Tensor<[1056]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1104, 14, 14]> input = ?",
            "Optional[Tensor]<[1104]> weight = ?",
            "Optional[Tensor]<[1104]> bias = ?",
            "Tensor<[1104]> running_mean = ?",
            "Tensor<[1104]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1152, 14, 14]> input = ?",
            "Optional[Tensor]<[1152]> weight = ?",
            "Optional[Tensor]<[1152]> bias = ?",
            "Tensor<[1152]> running_mean = ?",
            "Tensor<[1152]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1200, 14, 14]> input = ?",
            "Optional[Tensor]<[1200]> weight = ?",
            "Optional[Tensor]<[1200]> bias = ?",
            "Tensor<[1200]> running_mean = ?",
            "Tensor<[1200]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1248, 14, 14]> input = ?",
            "Optional[Tensor]<[1248]> weight = ?",
            "Optional[Tensor]<[1248]> bias = ?",
            "Tensor<[1248]> running_mean = ?",
            "Tensor<[1248]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1296, 14, 14]> input = ?",
            "Optional[Tensor]<[1296]> weight = ?",
            "Optional[Tensor]<[1296]> bias = ?",
            "Tensor<[1296]> running_mean = ?",
            "Tensor<[1296]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1344, 14, 14]> input = ?",
            "Optional[Tensor]<[1344]> weight = ?",
            "Optional[Tensor]<[1344]> bias = ?",
            "Tensor<[1344]> running_mean = ?",
            "Tensor<[1344]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1392, 14, 14]> input = ?",
            "Optional[Tensor]<[1392]> weight = ?",
            "Optional[Tensor]<[1392]> bias = ?",
            "Tensor<[1392]> running_mean = ?",
            "Tensor<[1392]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1440, 14, 14]> input = ?",
            "Optional[Tensor]<[1440]> weight = ?",
            "Optional[Tensor]<[1440]> bias = ?",
            "Tensor<[1440]> running_mean = ?",
            "Tensor<[1440]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1488, 14, 14]> input = ?",
            "Optional[Tensor]<[1488]> weight = ?",
            "Optional[Tensor]<[1488]> bias = ?",
            "Tensor<[1488]> running_mean = ?",
            "Tensor<[1488]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1536, 14, 14]> input = ?",
            "Optional[Tensor]<[1536]> weight = ?",
            "Optional[Tensor]<[1536]> bias = ?",
            "Tensor<[1536]> running_mean = ?",
            "Tensor<[1536]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1584, 14, 14]> input = ?",
            "Optional[Tensor]<[1584]> weight = ?",
            "Optional[Tensor]<[1584]> bias = ?",
            "Tensor<[1584]> running_mean = ?",
            "Tensor<[1584]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1632, 14, 14]> input = ?",
            "Optional[Tensor]<[1632]> weight = ?",
            "Optional[Tensor]<[1632]> bias = ?",
            "Tensor<[1632]> running_mean = ?",
            "Tensor<[1632]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1680, 14, 14]> input = ?",
            "Optional[Tensor]<[1680]> weight = ?",
            "Optional[Tensor]<[1680]> bias = ?",
            "Tensor<[1680]> running_mean = ?",
            "Tensor<[1680]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1728, 14, 14]> input = ?",
            "Optional[Tensor]<[1728]> weight = ?",
            "Optional[Tensor]<[1728]> bias = ?",
            "Tensor<[1728]> running_mean = ?",
            "Tensor<[1728]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1776, 14, 14]> input = ?",
            "Optional[Tensor]<[1776]> weight = ?",
            "Optional[Tensor]<[1776]> bias = ?",
            "Tensor<[1776]> running_mean = ?",
            "Tensor<[1776]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1824, 14, 14]> input = ?",
            "Optional[Tensor]<[1824]> weight = ?",
            "Optional[Tensor]<[1824]> bias = ?",
            "Tensor<[1824]> running_mean = ?",
            "Tensor<[1824]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1872, 14, 14]> input = ?",
            "Optional[Tensor]<[1872]> weight = ?",
            "Optional[Tensor]<[1872]> bias = ?",
            "Tensor<[1872]> running_mean = ?",
            "Tensor<[1872]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1920, 14, 14]> input = ?",
            "Optional[Tensor]<[1920]> weight = ?",
            "Optional[Tensor]<[1920]> bias = ?",
            "Tensor<[1920]> running_mean = ?",
            "Tensor<[1920]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1968, 14, 14]> input = ?",
            "Optional[Tensor]<[1968]> weight = ?",
            "Optional[Tensor]<[1968]> bias = ?",
            "Tensor<[1968]> running_mean = ?",
            "Tensor<[1968]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2016, 14, 14]> input = ?",
            "Optional[Tensor]<[2016]> weight = ?",
            "Optional[Tensor]<[2016]> bias = ?",
            "Tensor<[2016]> running_mean = ?",
            "Tensor<[2016]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2064, 14, 14]> input = ?",
            "Optional[Tensor]<[2064]> weight = ?",
            "Optional[Tensor]<[2064]> bias = ?",
            "Tensor<[2064]> running_mean = ?",
            "Tensor<[2064]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2112, 14, 14]> input = ?",
            "Optional[Tensor]<[2112]> weight = ?",
            "Optional[Tensor]<[2112]> bias = ?",
            "Tensor<[2112]> running_mean = ?",
            "Tensor<[2112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1056, 7, 7]> input = ?",
            "Optional[Tensor]<[1056]> weight = ?",
            "Optional[Tensor]<[1056]> bias = ?",
            "Tensor<[1056]> running_mean = ?",
            "Tensor<[1056]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 192, 7, 7]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1104, 7, 7]> input = ?",
            "Optional[Tensor]<[1104]> weight = ?",
            "Optional[Tensor]<[1104]> bias = ?",
            "Tensor<[1104]> running_mean = ?",
            "Tensor<[1104]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Optional[Tensor]<[1152]> weight = ?",
            "Optional[Tensor]<[1152]> bias = ?",
            "Tensor<[1152]> running_mean = ?",
            "Tensor<[1152]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1200, 7, 7]> input = ?",
            "Optional[Tensor]<[1200]> weight = ?",
            "Optional[Tensor]<[1200]> bias = ?",
            "Tensor<[1200]> running_mean = ?",
            "Tensor<[1200]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1248, 7, 7]> input = ?",
            "Optional[Tensor]<[1248]> weight = ?",
            "Optional[Tensor]<[1248]> bias = ?",
            "Tensor<[1248]> running_mean = ?",
            "Tensor<[1248]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1296, 7, 7]> input = ?",
            "Optional[Tensor]<[1296]> weight = ?",
            "Optional[Tensor]<[1296]> bias = ?",
            "Tensor<[1296]> running_mean = ?",
            "Tensor<[1296]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1344, 7, 7]> input = ?",
            "Optional[Tensor]<[1344]> weight = ?",
            "Optional[Tensor]<[1344]> bias = ?",
            "Tensor<[1344]> running_mean = ?",
            "Tensor<[1344]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1392, 7, 7]> input = ?",
            "Optional[Tensor]<[1392]> weight = ?",
            "Optional[Tensor]<[1392]> bias = ?",
            "Tensor<[1392]> running_mean = ?",
            "Tensor<[1392]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1440, 7, 7]> input = ?",
            "Optional[Tensor]<[1440]> weight = ?",
            "Optional[Tensor]<[1440]> bias = ?",
            "Tensor<[1440]> running_mean = ?",
            "Tensor<[1440]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1488, 7, 7]> input = ?",
            "Optional[Tensor]<[1488]> weight = ?",
            "Optional[Tensor]<[1488]> bias = ?",
            "Tensor<[1488]> running_mean = ?",
            "Tensor<[1488]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1536, 7, 7]> input = ?",
            "Optional[Tensor]<[1536]> weight = ?",
            "Optional[Tensor]<[1536]> bias = ?",
            "Tensor<[1536]> running_mean = ?",
            "Tensor<[1536]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1584, 7, 7]> input = ?",
            "Optional[Tensor]<[1584]> weight = ?",
            "Optional[Tensor]<[1584]> bias = ?",
            "Tensor<[1584]> running_mean = ?",
            "Tensor<[1584]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1632, 7, 7]> input = ?",
            "Optional[Tensor]<[1632]> weight = ?",
            "Optional[Tensor]<[1632]> bias = ?",
            "Tensor<[1632]> running_mean = ?",
            "Tensor<[1632]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1680, 7, 7]> input = ?",
            "Optional[Tensor]<[1680]> weight = ?",
            "Optional[Tensor]<[1680]> bias = ?",
            "Tensor<[1680]> running_mean = ?",
            "Tensor<[1680]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1728, 7, 7]> input = ?",
            "Optional[Tensor]<[1728]> weight = ?",
            "Optional[Tensor]<[1728]> bias = ?",
            "Tensor<[1728]> running_mean = ?",
            "Tensor<[1728]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1776, 7, 7]> input = ?",
            "Optional[Tensor]<[1776]> weight = ?",
            "Optional[Tensor]<[1776]> bias = ?",
            "Tensor<[1776]> running_mean = ?",
            "Tensor<[1776]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1824, 7, 7]> input = ?",
            "Optional[Tensor]<[1824]> weight = ?",
            "Optional[Tensor]<[1824]> bias = ?",
            "Tensor<[1824]> running_mean = ?",
            "Tensor<[1824]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1872, 7, 7]> input = ?",
            "Optional[Tensor]<[1872]> weight = ?",
            "Optional[Tensor]<[1872]> bias = ?",
            "Tensor<[1872]> running_mean = ?",
            "Tensor<[1872]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1920, 7, 7]> input = ?",
            "Optional[Tensor]<[1920]> weight = ?",
            "Optional[Tensor]<[1920]> bias = ?",
            "Tensor<[1920]> running_mean = ?",
            "Tensor<[1920]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1968, 7, 7]> input = ?",
            "Optional[Tensor]<[1968]> weight = ?",
            "Optional[Tensor]<[1968]> bias = ?",
            "Tensor<[1968]> running_mean = ?",
            "Tensor<[1968]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2016, 7, 7]> input = ?",
            "Optional[Tensor]<[2016]> weight = ?",
            "Optional[Tensor]<[2016]> bias = ?",
            "Tensor<[2016]> running_mean = ?",
            "Tensor<[2016]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2064, 7, 7]> input = ?",
            "Optional[Tensor]<[2064]> weight = ?",
            "Optional[Tensor]<[2064]> bias = ?",
            "Tensor<[2064]> running_mean = ?",
            "Tensor<[2064]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2112, 7, 7]> input = ?",
            "Optional[Tensor]<[2112]> weight = ?",
            "Optional[Tensor]<[2112]> bias = ?",
            "Tensor<[2112]> running_mean = ?",
            "Tensor<[2112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2160, 7, 7]> input = ?",
            "Optional[Tensor]<[2160]> weight = ?",
            "Optional[Tensor]<[2160]> bias = ?",
            "Tensor<[2160]> running_mean = ?",
            "Tensor<[2160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2208, 7, 7]> input = ?",
            "Optional[Tensor]<[2208]> weight = ?",
            "Optional[Tensor]<[2208]> bias = ?",
            "Tensor<[2208]> running_mean = ?",
            "Tensor<[2208]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1088, 14, 14]> input = ?",
            "Optional[Tensor]<[1088]> weight = ?",
            "Optional[Tensor]<[1088]> bias = ?",
            "Tensor<[1088]> running_mean = ?",
            "Tensor<[1088]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1120, 14, 14]> input = ?",
            "Optional[Tensor]<[1120]> weight = ?",
            "Optional[Tensor]<[1120]> bias = ?",
            "Tensor<[1120]> running_mean = ?",
            "Tensor<[1120]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1184, 14, 14]> input = ?",
            "Optional[Tensor]<[1184]> weight = ?",
            "Optional[Tensor]<[1184]> bias = ?",
            "Tensor<[1184]> running_mean = ?",
            "Tensor<[1184]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1216, 14, 14]> input = ?",
            "Optional[Tensor]<[1216]> weight = ?",
            "Optional[Tensor]<[1216]> bias = ?",
            "Tensor<[1216]> running_mean = ?",
            "Tensor<[1216]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1280, 14, 14]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1088, 7, 7]> input = ?",
            "Optional[Tensor]<[1088]> weight = ?",
            "Optional[Tensor]<[1088]> bias = ?",
            "Tensor<[1088]> running_mean = ?",
            "Tensor<[1088]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1120, 7, 7]> input = ?",
            "Optional[Tensor]<[1120]> weight = ?",
            "Optional[Tensor]<[1120]> bias = ?",
            "Tensor<[1120]> running_mean = ?",
            "Tensor<[1120]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1184, 7, 7]> input = ?",
            "Optional[Tensor]<[1184]> weight = ?",
            "Optional[Tensor]<[1184]> bias = ?",
            "Tensor<[1184]> running_mean = ?",
            "Tensor<[1184]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1216, 7, 7]> input = ?",
            "Optional[Tensor]<[1216]> weight = ?",
            "Optional[Tensor]<[1216]> bias = ?",
            "Tensor<[1216]> running_mean = ?",
            "Tensor<[1216]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1312, 7, 7]> input = ?",
            "Optional[Tensor]<[1312]> weight = ?",
            "Optional[Tensor]<[1312]> bias = ?",
            "Tensor<[1312]> running_mean = ?",
            "Tensor<[1312]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1376, 7, 7]> input = ?",
            "Optional[Tensor]<[1376]> weight = ?",
            "Optional[Tensor]<[1376]> bias = ?",
            "Tensor<[1376]> running_mean = ?",
            "Tensor<[1376]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1408, 7, 7]> input = ?",
            "Optional[Tensor]<[1408]> weight = ?",
            "Optional[Tensor]<[1408]> bias = ?",
            "Tensor<[1408]> running_mean = ?",
            "Tensor<[1408]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1472, 7, 7]> input = ?",
            "Optional[Tensor]<[1472]> weight = ?",
            "Optional[Tensor]<[1472]> bias = ?",
            "Tensor<[1472]> running_mean = ?",
            "Tensor<[1472]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1504, 7, 7]> input = ?",
            "Optional[Tensor]<[1504]> weight = ?",
            "Optional[Tensor]<[1504]> bias = ?",
            "Tensor<[1504]> running_mean = ?",
            "Tensor<[1504]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1568, 7, 7]> input = ?",
            "Optional[Tensor]<[1568]> weight = ?",
            "Optional[Tensor]<[1568]> bias = ?",
            "Tensor<[1568]> running_mean = ?",
            "Tensor<[1568]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1600, 7, 7]> input = ?",
            "Optional[Tensor]<[1600]> weight = ?",
            "Optional[Tensor]<[1600]> bias = ?",
            "Tensor<[1600]> running_mean = ?",
            "Tensor<[1600]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1664, 7, 7]> input = ?",
            "Optional[Tensor]<[1664]> weight = ?",
            "Optional[Tensor]<[1664]> bias = ?",
            "Tensor<[1664]> running_mean = ?",
            "Tensor<[1664]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1312, 14, 14]> input = ?",
            "Optional[Tensor]<[1312]> weight = ?",
            "Optional[Tensor]<[1312]> bias = ?",
            "Tensor<[1312]> running_mean = ?",
            "Tensor<[1312]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1376, 14, 14]> input = ?",
            "Optional[Tensor]<[1376]> weight = ?",
            "Optional[Tensor]<[1376]> bias = ?",
            "Tensor<[1376]> running_mean = ?",
            "Tensor<[1376]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1408, 14, 14]> input = ?",
            "Optional[Tensor]<[1408]> weight = ?",
            "Optional[Tensor]<[1408]> bias = ?",
            "Tensor<[1408]> running_mean = ?",
            "Tensor<[1408]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1472, 14, 14]> input = ?",
            "Optional[Tensor]<[1472]> weight = ?",
            "Optional[Tensor]<[1472]> bias = ?",
            "Tensor<[1472]> running_mean = ?",
            "Tensor<[1472]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1504, 14, 14]> input = ?",
            "Optional[Tensor]<[1504]> weight = ?",
            "Optional[Tensor]<[1504]> bias = ?",
            "Tensor<[1504]> running_mean = ?",
            "Tensor<[1504]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1568, 14, 14]> input = ?",
            "Optional[Tensor]<[1568]> weight = ?",
            "Optional[Tensor]<[1568]> bias = ?",
            "Tensor<[1568]> running_mean = ?",
            "Tensor<[1568]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1600, 14, 14]> input = ?",
            "Optional[Tensor]<[1600]> weight = ?",
            "Optional[Tensor]<[1600]> bias = ?",
            "Tensor<[1600]> running_mean = ?",
            "Tensor<[1600]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1664, 14, 14]> input = ?",
            "Optional[Tensor]<[1664]> weight = ?",
            "Optional[Tensor]<[1664]> bias = ?",
            "Tensor<[1664]> running_mean = ?",
            "Tensor<[1664]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1696, 14, 14]> input = ?",
            "Optional[Tensor]<[1696]> weight = ?",
            "Optional[Tensor]<[1696]> bias = ?",
            "Tensor<[1696]> running_mean = ?",
            "Tensor<[1696]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1760, 14, 14]> input = ?",
            "Optional[Tensor]<[1760]> weight = ?",
            "Optional[Tensor]<[1760]> bias = ?",
            "Tensor<[1760]> running_mean = ?",
            "Tensor<[1760]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1792, 14, 14]> input = ?",
            "Optional[Tensor]<[1792]> weight = ?",
            "Optional[Tensor]<[1792]> bias = ?",
            "Tensor<[1792]> running_mean = ?",
            "Tensor<[1792]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1696, 7, 7]> input = ?",
            "Optional[Tensor]<[1696]> weight = ?",
            "Optional[Tensor]<[1696]> bias = ?",
            "Tensor<[1696]> running_mean = ?",
            "Tensor<[1696]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1760, 7, 7]> input = ?",
            "Optional[Tensor]<[1760]> weight = ?",
            "Optional[Tensor]<[1760]> bias = ?",
            "Tensor<[1760]> running_mean = ?",
            "Tensor<[1760]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1792, 7, 7]> input = ?",
            "Optional[Tensor]<[1792]> weight = ?",
            "Optional[Tensor]<[1792]> bias = ?",
            "Tensor<[1792]> running_mean = ?",
            "Tensor<[1792]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1856, 7, 7]> input = ?",
            "Optional[Tensor]<[1856]> weight = ?",
            "Optional[Tensor]<[1856]> bias = ?",
            "Tensor<[1856]> running_mean = ?",
            "Tensor<[1856]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1888, 7, 7]> input = ?",
            "Optional[Tensor]<[1888]> weight = ?",
            "Optional[Tensor]<[1888]> bias = ?",
            "Tensor<[1888]> running_mean = ?",
            "Tensor<[1888]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 16, 224, 224]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 224, 7, 7]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 8, 112, 112]> input = ?",
            "Optional[Tensor]<[8]> weight = ?",
            "Optional[Tensor]<[8]> bias = ?",
            "Tensor<[8]> running_mean = ?",
            "Tensor<[8]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 24, 112, 112]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 48, 56, 56]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 12, 56, 56]> input = ?",
            "Optional[Tensor]<[12]> weight = ?",
            "Optional[Tensor]<[12]> bias = ?",
            "Tensor<[12]> running_mean = ?",
            "Tensor<[12]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 16, 56, 56]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 36, 56, 56]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> bias = ?",
            "Tensor<[36]> running_mean = ?",
            "Tensor<[36]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 60, 28, 28]> input = ?",
            "Optional[Tensor]<[60]> weight = ?",
            "Optional[Tensor]<[60]> bias = ?",
            "Tensor<[60]> running_mean = ?",
            "Tensor<[60]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 100, 14, 14]> input = ?",
            "Optional[Tensor]<[100]> weight = ?",
            "Optional[Tensor]<[100]> bias = ?",
            "Tensor<[100]> running_mean = ?",
            "Tensor<[100]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 92, 14, 14]> input = ?",
            "Optional[Tensor]<[92]> weight = ?",
            "Optional[Tensor]<[92]> bias = ?",
            "Tensor<[92]> running_mean = ?",
            "Tensor<[92]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 56, 14, 14]> input = ?",
            "Optional[Tensor]<[56]> weight = ?",
            "Optional[Tensor]<[56]> bias = ?",
            "Tensor<[56]> running_mean = ?",
            "Tensor<[56]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 336, 14, 14]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> bias = ?",
            "Tensor<[336]> running_mean = ?",
            "Tensor<[336]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 80, 7, 7]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 112, 7, 7]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 480, 7, 7]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 200, 7, 7]> input = ?",
            "Optional[Tensor]<[200]> weight = ?",
            "Optional[Tensor]<[200]> bias = ?",
            "Tensor<[200]> running_mean = ?",
            "Tensor<[200]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 184, 7, 7]> input = ?",
            "Optional[Tensor]<[184]> weight = ?",
            "Optional[Tensor]<[184]> bias = ?",
            "Tensor<[184]> running_mean = ?",
            "Tensor<[184]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 960, 3, 3]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> bias = ?",
            "Tensor<[960]> running_mean = ?",
            "Tensor<[960]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 56, 56]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 28, 28]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 28, 28]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 28, 28]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 28, 28]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 28, 28]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 28, 28]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 14, 14]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 14, 14]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 208, 14, 14]> input = ?",
            "Optional[Tensor]<[208]> weight = ?",
            "Optional[Tensor]<[208]> bias = ?",
            "Tensor<[208]> running_mean = ?",
            "Tensor<[208]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 14, 14]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 48, 14, 14]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 14, 14]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 160, 14, 14]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 224, 14, 14]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 14, 14]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 14, 14]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 14, 14]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 14, 14]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 14, 14]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 14, 14]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 320, 14, 14]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 7, 7]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 320, 7, 7]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 7, 7]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 7, 7]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 384, 7, 7]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 7, 7]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 48, 7, 7]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 18, 56, 56]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> bias = ?",
            "Tensor<[18]> running_mean = ?",
            "Tensor<[18]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 36, 28, 28]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> bias = ?",
            "Tensor<[36]> running_mean = ?",
            "Tensor<[36]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 18, 28, 28]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> bias = ?",
            "Tensor<[18]> running_mean = ?",
            "Tensor<[18]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 72, 14, 14]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 18, 14, 14]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> bias = ?",
            "Tensor<[18]> running_mean = ?",
            "Tensor<[18]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 36, 14, 14]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> bias = ?",
            "Tensor<[36]> running_mean = ?",
            "Tensor<[36]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 144, 7, 7]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 18, 7, 7]> input = ?",
            "Optional[Tensor]<[18]> weight = ?",
            "Optional[Tensor]<[18]> bias = ?",
            "Tensor<[18]> running_mean = ?",
            "Tensor<[18]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 36, 7, 7]> input = ?",
            "Optional[Tensor]<[36]> weight = ?",
            "Optional[Tensor]<[36]> bias = ?",
            "Tensor<[36]> running_mean = ?",
            "Tensor<[36]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 72, 7, 7]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 56, 56]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 28, 28]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 7, 7]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 149, 149]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 147, 147]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 147, 147]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 73, 73]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 73, 73]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 71, 71]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 35, 35]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 35, 35]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 35, 35]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 384, 17, 17]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 224, 35, 35]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 17, 17]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 17, 17]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 224, 17, 17]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 17, 17]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 8, 8]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 320, 17, 17]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 320, 8, 8]> input = ?",
            "Optional[Tensor]<[320]> weight = ?",
            "Optional[Tensor]<[320]> bias = ?",
            "Tensor<[320]> running_mean = ?",
            "Tensor<[320]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 8, 8]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 384, 8, 8]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 448, 8, 8]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> bias = ?",
            "Tensor<[448]> running_mean = ?",
            "Tensor<[448]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 512, 8, 8]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 112, 112]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 56, 56]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 72, 56, 56]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 72, 28, 28]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 120, 28, 28]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 200, 14, 14]> input = ?",
            "Optional[Tensor]<[200]> weight = ?",
            "Optional[Tensor]<[200]> bias = ?",
            "Tensor<[200]> running_mean = ?",
            "Tensor<[200]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 184, 14, 14]> input = ?",
            "Optional[Tensor]<[184]> weight = ?",
            "Optional[Tensor]<[184]> bias = ?",
            "Tensor<[184]> running_mean = ?",
            "Tensor<[184]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 112, 14, 14]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 160, 7, 7]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 960, 7, 7]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> bias = ?",
            "Tensor<[960]> running_mean = ?",
            "Tensor<[960]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 56, 56]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 28, 28]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 88, 28, 28]> input = ?",
            "Optional[Tensor]<[88]> weight = ?",
            "Optional[Tensor]<[88]> bias = ?",
            "Tensor<[88]> running_mean = ?",
            "Tensor<[88]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 28, 28]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 14, 14]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 40, 14, 14]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 120, 14, 14]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 48, 14, 14]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 14, 14]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 14, 14]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 7, 7]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 7, 7]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 576, 7, 7]> input = ?",
            "Optional[Tensor]<[576]> weight = ?",
            "Optional[Tensor]<[576]> bias = ?",
            "Tensor<[576]> running_mean = ?",
            "Tensor<[576]> running_var = ?",
            "float momentum = 0.01",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 112, 112]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 56, 56]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 896, 28, 28]> input = ?",
            "Optional[Tensor]<[896]> weight = ?",
            "Optional[Tensor]<[896]> bias = ?",
            "Tensor<[896]> running_mean = ?",
            "Tensor<[896]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2048, 14, 14]> input = ?",
            "Optional[Tensor]<[2048]> weight = ?",
            "Optional[Tensor]<[2048]> bias = ?",
            "Tensor<[2048]> running_mean = ?",
            "Tensor<[2048]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 72, 56, 56]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 72, 112, 112]> input = ?",
            "Optional[Tensor]<[72]> weight = ?",
            "Optional[Tensor]<[72]> bias = ?",
            "Tensor<[72]> running_mean = ?",
            "Tensor<[72]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 168, 56, 56]> input = ?",
            "Optional[Tensor]<[168]> weight = ?",
            "Optional[Tensor]<[168]> bias = ?",
            "Tensor<[168]> running_mean = ?",
            "Tensor<[168]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 408, 14, 14]> input = ?",
            "Optional[Tensor]<[408]> weight = ?",
            "Optional[Tensor]<[408]> bias = ?",
            "Tensor<[408]> running_mean = ?",
            "Tensor<[408]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 408, 28, 28]> input = ?",
            "Optional[Tensor]<[408]> weight = ?",
            "Optional[Tensor]<[408]> bias = ?",
            "Tensor<[408]> running_mean = ?",
            "Tensor<[408]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 912, 7, 7]> input = ?",
            "Optional[Tensor]<[912]> weight = ?",
            "Optional[Tensor]<[912]> bias = ?",
            "Tensor<[912]> running_mean = ?",
            "Tensor<[912]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 336, 112, 112]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> bias = ?",
            "Tensor<[336]> running_mean = ?",
            "Tensor<[336]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 672, 56, 56]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1344, 28, 28]> input = ?",
            "Optional[Tensor]<[1344]> weight = ?",
            "Optional[Tensor]<[1344]> bias = ?",
            "Tensor<[1344]> running_mean = ?",
            "Tensor<[1344]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2520, 7, 7]> input = ?",
            "Optional[Tensor]<[2520]> weight = ?",
            "Optional[Tensor]<[2520]> bias = ?",
            "Tensor<[2520]> running_mean = ?",
            "Tensor<[2520]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2520, 14, 14]> input = ?",
            "Optional[Tensor]<[2520]> weight = ?",
            "Optional[Tensor]<[2520]> bias = ?",
            "Tensor<[2520]> running_mean = ?",
            "Tensor<[2520]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1008, 7, 7]> input = ?",
            "Optional[Tensor]<[1008]> weight = ?",
            "Optional[Tensor]<[1008]> bias = ?",
            "Tensor<[1008]> running_mean = ?",
            "Tensor<[1008]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 160, 14, 14]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 400, 7, 7]> input = ?",
            "Optional[Tensor]<[400]> weight = ?",
            "Optional[Tensor]<[400]> bias = ?",
            "Tensor<[400]> running_mean = ?",
            "Tensor<[400]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 400, 14, 14]> input = ?",
            "Optional[Tensor]<[400]> weight = ?",
            "Optional[Tensor]<[400]> bias = ?",
            "Tensor<[400]> running_mean = ?",
            "Tensor<[400]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 80, 56, 56]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 80, 112, 112]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 192, 192]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 528, 96, 96]> input = ?",
            "Optional[Tensor]<[528]> weight = ?",
            "Optional[Tensor]<[528]> bias = ?",
            "Tensor<[528]> running_mean = ?",
            "Tensor<[528]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 528, 192, 192]> input = ?",
            "Optional[Tensor]<[528]> weight = ?",
            "Optional[Tensor]<[528]> bias = ?",
            "Tensor<[528]> running_mean = ?",
            "Tensor<[528]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1056, 48, 48]> input = ?",
            "Optional[Tensor]<[1056]> weight = ?",
            "Optional[Tensor]<[1056]> bias = ?",
            "Tensor<[1056]> running_mean = ?",
            "Tensor<[1056]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1056, 96, 96]> input = ?",
            "Optional[Tensor]<[1056]> weight = ?",
            "Optional[Tensor]<[1056]> bias = ?",
            "Tensor<[1056]> running_mean = ?",
            "Tensor<[1056]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2904, 24, 24]> input = ?",
            "Optional[Tensor]<[2904]> weight = ?",
            "Optional[Tensor]<[2904]> bias = ?",
            "Tensor<[2904]> running_mean = ?",
            "Tensor<[2904]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2904, 48, 48]> input = ?",
            "Optional[Tensor]<[2904]> weight = ?",
            "Optional[Tensor]<[2904]> bias = ?",
            "Tensor<[2904]> running_mean = ?",
            "Tensor<[2904]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 7392, 12, 12]> input = ?",
            "Optional[Tensor]<[7392]> weight = ?",
            "Optional[Tensor]<[7392]> bias = ?",
            "Tensor<[7392]> running_mean = ?",
            "Tensor<[7392]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 7392, 24, 24]> input = ?",
            "Optional[Tensor]<[7392]> weight = ?",
            "Optional[Tensor]<[7392]> bias = ?",
            "Tensor<[7392]> running_mean = ?",
            "Tensor<[7392]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 224, 112, 112]> input = ?",
            "Optional[Tensor]<[224]> weight = ?",
            "Optional[Tensor]<[224]> bias = ?",
            "Tensor<[224]> running_mean = ?",
            "Tensor<[224]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 448, 56, 56]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> bias = ?",
            "Tensor<[448]> running_mean = ?",
            "Tensor<[448]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1232, 14, 14]> input = ?",
            "Optional[Tensor]<[1232]> weight = ?",
            "Optional[Tensor]<[1232]> bias = ?",
            "Tensor<[1232]> running_mean = ?",
            "Tensor<[1232]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1232, 28, 28]> input = ?",
            "Optional[Tensor]<[1232]> weight = ?",
            "Optional[Tensor]<[1232]> bias = ?",
            "Tensor<[1232]> running_mean = ?",
            "Tensor<[1232]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 3024, 7, 7]> input = ?",
            "Optional[Tensor]<[3024]> weight = ?",
            "Optional[Tensor]<[3024]> bias = ?",
            "Tensor<[3024]> running_mean = ?",
            "Tensor<[3024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 3024, 14, 14]> input = ?",
            "Optional[Tensor]<[3024]> weight = ?",
            "Optional[Tensor]<[3024]> bias = ?",
            "Tensor<[3024]> running_mean = ?",
            "Tensor<[3024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 48, 112, 112]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 120, 56, 56]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 888, 7, 7]> input = ?",
            "Optional[Tensor]<[888]> weight = ?",
            "Optional[Tensor]<[888]> bias = ?",
            "Tensor<[888]> running_mean = ?",
            "Tensor<[888]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 888, 14, 14]> input = ?",
            "Optional[Tensor]<[888]> weight = ?",
            "Optional[Tensor]<[888]> bias = ?",
            "Tensor<[888]> running_mean = ?",
            "Tensor<[888]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 232, 56, 56]> input = ?",
            "Optional[Tensor]<[232]> weight = ?",
            "Optional[Tensor]<[232]> bias = ?",
            "Tensor<[232]> running_mean = ?",
            "Tensor<[232]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 232, 112, 112]> input = ?",
            "Optional[Tensor]<[232]> weight = ?",
            "Optional[Tensor]<[232]> bias = ?",
            "Tensor<[232]> running_mean = ?",
            "Tensor<[232]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 696, 28, 28]> input = ?",
            "Optional[Tensor]<[696]> weight = ?",
            "Optional[Tensor]<[696]> bias = ?",
            "Tensor<[696]> running_mean = ?",
            "Tensor<[696]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 696, 56, 56]> input = ?",
            "Optional[Tensor]<[696]> weight = ?",
            "Optional[Tensor]<[696]> bias = ?",
            "Tensor<[696]> running_mean = ?",
            "Tensor<[696]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1392, 28, 28]> input = ?",
            "Optional[Tensor]<[1392]> weight = ?",
            "Optional[Tensor]<[1392]> bias = ?",
            "Tensor<[1392]> running_mean = ?",
            "Tensor<[1392]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 3712, 7, 7]> input = ?",
            "Optional[Tensor]<[3712]> weight = ?",
            "Optional[Tensor]<[3712]> bias = ?",
            "Tensor<[3712]> running_mean = ?",
            "Tensor<[3712]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 3712, 14, 14]> input = ?",
            "Optional[Tensor]<[3712]> weight = ?",
            "Optional[Tensor]<[3712]> bias = ?",
            "Tensor<[3712]> running_mean = ?",
            "Tensor<[3712]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 216, 28, 28]> input = ?",
            "Optional[Tensor]<[216]> weight = ?",
            "Optional[Tensor]<[216]> bias = ?",
            "Tensor<[216]> running_mean = ?",
            "Tensor<[216]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 216, 56, 56]> input = ?",
            "Optional[Tensor]<[216]> weight = ?",
            "Optional[Tensor]<[216]> bias = ?",
            "Tensor<[216]> running_mean = ?",
            "Tensor<[216]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1512, 7, 7]> input = ?",
            "Optional[Tensor]<[1512]> weight = ?",
            "Optional[Tensor]<[1512]> bias = ?",
            "Tensor<[1512]> running_mean = ?",
            "Tensor<[1512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1512, 14, 14]> input = ?",
            "Optional[Tensor]<[1512]> weight = ?",
            "Optional[Tensor]<[1512]> bias = ?",
            "Tensor<[1512]> running_mean = ?",
            "Tensor<[1512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 104, 28, 28]> input = ?",
            "Optional[Tensor]<[104]> weight = ?",
            "Optional[Tensor]<[104]> bias = ?",
            "Tensor<[104]> running_mean = ?",
            "Tensor<[104]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 104, 56, 56]> input = ?",
            "Optional[Tensor]<[104]> weight = ?",
            "Optional[Tensor]<[104]> bias = ?",
            "Tensor<[104]> running_mean = ?",
            "Tensor<[104]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 208, 14, 14]> input = ?",
            "Optional[Tensor]<[208]> weight = ?",
            "Optional[Tensor]<[208]> bias = ?",
            "Tensor<[208]> running_mean = ?",
            "Tensor<[208]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 208, 28, 28]> input = ?",
            "Optional[Tensor]<[208]> weight = ?",
            "Optional[Tensor]<[208]> bias = ?",
            "Tensor<[208]> running_mean = ?",
            "Tensor<[208]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 440, 7, 7]> input = ?",
            "Optional[Tensor]<[440]> weight = ?",
            "Optional[Tensor]<[440]> bias = ?",
            "Tensor<[440]> running_mean = ?",
            "Tensor<[440]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 440, 14, 14]> input = ?",
            "Optional[Tensor]<[440]> weight = ?",
            "Optional[Tensor]<[440]> bias = ?",
            "Tensor<[440]> running_mean = ?",
            "Tensor<[440]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 784, 7, 7]> input = ?",
            "Optional[Tensor]<[784]> weight = ?",
            "Optional[Tensor]<[784]> bias = ?",
            "Tensor<[784]> running_mean = ?",
            "Tensor<[784]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 784, 14, 14]> input = ?",
            "Optional[Tensor]<[784]> weight = ?",
            "Optional[Tensor]<[784]> bias = ?",
            "Tensor<[784]> running_mean = ?",
            "Tensor<[784]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1024, 28, 28]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 400, 544]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 64, 200, 272]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 200, 272]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 200, 272]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 128, 100, 136]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 100, 136]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 100, 136]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 256, 50, 68]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 1024, 50, 68]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 50, 68]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 512, 25, 34]> input = ?",
            "Optional[Tensor]<[512]> weight = ?",
            "Optional[Tensor]<[512]> bias = ?",
            "Tensor<[512]> running_mean = ?",
            "Tensor<[512]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 2048, 25, 34]> input = ?",
            "Optional[Tensor]<[2048]> weight = ?",
            "Optional[Tensor]<[2048]> bias = ?",
            "Tensor<[2048]> running_mean = ?",
            "Tensor<[2048]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 1e-05",
        ],
        [
            "Tensor<[1, 32, 112, 112]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 112, 112]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 112, 112]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 56, 56]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 56, 56]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 56, 56]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 28, 28]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 40, 28, 28]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 28, 28]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 14, 14]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 80, 14, 14]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 480, 14, 14]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 14, 14]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 7, 7]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1152, 7, 7]> input = ?",
            "Optional[Tensor]<[1152]> weight = ?",
            "Optional[Tensor]<[1152]> bias = ?",
            "Tensor<[1152]> running_mean = ?",
            "Tensor<[1152]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1280, 7, 7]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 120, 120]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 120, 120]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 120, 120]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 60, 60]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 60, 60]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 60, 60]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 30, 30]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 40, 30, 30]> input = ?",
            "Optional[Tensor]<[40]> weight = ?",
            "Optional[Tensor]<[40]> bias = ?",
            "Tensor<[40]> running_mean = ?",
            "Tensor<[40]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 30, 30]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 240, 15, 15]> input = ?",
            "Optional[Tensor]<[240]> weight = ?",
            "Optional[Tensor]<[240]> bias = ?",
            "Tensor<[240]> running_mean = ?",
            "Tensor<[240]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 80, 15, 15]> input = ?",
            "Optional[Tensor]<[80]> weight = ?",
            "Optional[Tensor]<[80]> bias = ?",
            "Tensor<[80]> running_mean = ?",
            "Tensor<[80]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 480, 15, 15]> input = ?",
            "Optional[Tensor]<[480]> weight = ?",
            "Optional[Tensor]<[480]> bias = ?",
            "Tensor<[480]> running_mean = ?",
            "Tensor<[480]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 112, 15, 15]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 15, 15]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 8, 8]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1152, 8, 8]> input = ?",
            "Optional[Tensor]<[1152]> weight = ?",
            "Optional[Tensor]<[1152]> bias = ?",
            "Tensor<[1152]> running_mean = ?",
            "Tensor<[1152]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1280, 8, 8]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 130, 130]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 16, 130, 130]> input = ?",
            "Optional[Tensor]<[16]> weight = ?",
            "Optional[Tensor]<[16]> bias = ?",
            "Tensor<[16]> running_mean = ?",
            "Tensor<[16]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 130, 130]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 65, 65]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 65, 65]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 65, 65]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 33, 33]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 48, 33, 33]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 33, 33]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 17, 17]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 88, 17, 17]> input = ?",
            "Optional[Tensor]<[88]> weight = ?",
            "Optional[Tensor]<[88]> bias = ?",
            "Tensor<[88]> running_mean = ?",
            "Tensor<[88]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 528, 17, 17]> input = ?",
            "Optional[Tensor]<[528]> weight = ?",
            "Optional[Tensor]<[528]> bias = ?",
            "Tensor<[528]> running_mean = ?",
            "Tensor<[528]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 120, 17, 17]> input = ?",
            "Optional[Tensor]<[120]> weight = ?",
            "Optional[Tensor]<[120]> bias = ?",
            "Tensor<[120]> running_mean = ?",
            "Tensor<[120]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 720, 17, 17]> input = ?",
            "Optional[Tensor]<[720]> weight = ?",
            "Optional[Tensor]<[720]> bias = ?",
            "Tensor<[720]> running_mean = ?",
            "Tensor<[720]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 720, 9, 9]> input = ?",
            "Optional[Tensor]<[720]> weight = ?",
            "Optional[Tensor]<[720]> bias = ?",
            "Tensor<[720]> running_mean = ?",
            "Tensor<[720]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 208, 9, 9]> input = ?",
            "Optional[Tensor]<[208]> weight = ?",
            "Optional[Tensor]<[208]> bias = ?",
            "Tensor<[208]> running_mean = ?",
            "Tensor<[208]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1248, 9, 9]> input = ?",
            "Optional[Tensor]<[1248]> weight = ?",
            "Optional[Tensor]<[1248]> bias = ?",
            "Tensor<[1248]> running_mean = ?",
            "Tensor<[1248]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 352, 9, 9]> input = ?",
            "Optional[Tensor]<[352]> weight = ?",
            "Optional[Tensor]<[352]> bias = ?",
            "Tensor<[352]> running_mean = ?",
            "Tensor<[352]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1280, 9, 9]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 150, 150]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 150, 150]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 150, 150]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 75, 75]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 75, 75]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 75, 75]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 38, 38]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 48, 38, 38]> input = ?",
            "Optional[Tensor]<[48]> weight = ?",
            "Optional[Tensor]<[48]> bias = ?",
            "Tensor<[48]> running_mean = ?",
            "Tensor<[48]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 38, 38]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 288, 19, 19]> input = ?",
            "Optional[Tensor]<[288]> weight = ?",
            "Optional[Tensor]<[288]> bias = ?",
            "Tensor<[288]> running_mean = ?",
            "Tensor<[288]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 96, 19, 19]> input = ?",
            "Optional[Tensor]<[96]> weight = ?",
            "Optional[Tensor]<[96]> bias = ?",
            "Tensor<[96]> running_mean = ?",
            "Tensor<[96]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 576, 19, 19]> input = ?",
            "Optional[Tensor]<[576]> weight = ?",
            "Optional[Tensor]<[576]> bias = ?",
            "Tensor<[576]> running_mean = ?",
            "Tensor<[576]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 136, 19, 19]> input = ?",
            "Optional[Tensor]<[136]> weight = ?",
            "Optional[Tensor]<[136]> bias = ?",
            "Tensor<[136]> running_mean = ?",
            "Tensor<[136]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 816, 19, 19]> input = ?",
            "Optional[Tensor]<[816]> weight = ?",
            "Optional[Tensor]<[816]> bias = ?",
            "Tensor<[816]> running_mean = ?",
            "Tensor<[816]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 816, 10, 10]> input = ?",
            "Optional[Tensor]<[816]> weight = ?",
            "Optional[Tensor]<[816]> bias = ?",
            "Tensor<[816]> running_mean = ?",
            "Tensor<[816]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 232, 10, 10]> input = ?",
            "Optional[Tensor]<[232]> weight = ?",
            "Optional[Tensor]<[232]> bias = ?",
            "Tensor<[232]> running_mean = ?",
            "Tensor<[232]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1392, 10, 10]> input = ?",
            "Optional[Tensor]<[1392]> weight = ?",
            "Optional[Tensor]<[1392]> bias = ?",
            "Tensor<[1392]> running_mean = ?",
            "Tensor<[1392]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 384, 10, 10]> input = ?",
            "Optional[Tensor]<[384]> weight = ?",
            "Optional[Tensor]<[384]> bias = ?",
            "Tensor<[384]> running_mean = ?",
            "Tensor<[384]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1280, 10, 10]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 190, 190]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 24, 190, 190]> input = ?",
            "Optional[Tensor]<[24]> weight = ?",
            "Optional[Tensor]<[24]> bias = ?",
            "Tensor<[24]> running_mean = ?",
            "Tensor<[24]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 190, 190]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 144, 95, 95]> input = ?",
            "Optional[Tensor]<[144]> weight = ?",
            "Optional[Tensor]<[144]> bias = ?",
            "Tensor<[144]> running_mean = ?",
            "Tensor<[144]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 32, 95, 95]> input = ?",
            "Optional[Tensor]<[32]> weight = ?",
            "Optional[Tensor]<[32]> bias = ?",
            "Tensor<[32]> running_mean = ?",
            "Tensor<[32]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 95, 95]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 192, 48, 48]> input = ?",
            "Optional[Tensor]<[192]> weight = ?",
            "Optional[Tensor]<[192]> bias = ?",
            "Tensor<[192]> running_mean = ?",
            "Tensor<[192]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 56, 48, 48]> input = ?",
            "Optional[Tensor]<[56]> weight = ?",
            "Optional[Tensor]<[56]> bias = ?",
            "Tensor<[56]> running_mean = ?",
            "Tensor<[56]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 336, 48, 48]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> bias = ?",
            "Tensor<[336]> running_mean = ?",
            "Tensor<[336]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 336, 24, 24]> input = ?",
            "Optional[Tensor]<[336]> weight = ?",
            "Optional[Tensor]<[336]> bias = ?",
            "Tensor<[336]> running_mean = ?",
            "Tensor<[336]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 112, 24, 24]> input = ?",
            "Optional[Tensor]<[112]> weight = ?",
            "Optional[Tensor]<[112]> bias = ?",
            "Tensor<[112]> running_mean = ?",
            "Tensor<[112]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 672, 24, 24]> input = ?",
            "Optional[Tensor]<[672]> weight = ?",
            "Optional[Tensor]<[672]> bias = ?",
            "Tensor<[672]> running_mean = ?",
            "Tensor<[672]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 160, 24, 24]> input = ?",
            "Optional[Tensor]<[160]> weight = ?",
            "Optional[Tensor]<[160]> bias = ?",
            "Tensor<[160]> running_mean = ?",
            "Tensor<[160]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 960, 24, 24]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> bias = ?",
            "Tensor<[960]> running_mean = ?",
            "Tensor<[960]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 960, 12, 12]> input = ?",
            "Optional[Tensor]<[960]> weight = ?",
            "Optional[Tensor]<[960]> bias = ?",
            "Tensor<[960]> running_mean = ?",
            "Tensor<[960]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 272, 12, 12]> input = ?",
            "Optional[Tensor]<[272]> weight = ?",
            "Optional[Tensor]<[272]> bias = ?",
            "Tensor<[272]> running_mean = ?",
            "Tensor<[272]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1632, 12, 12]> input = ?",
            "Optional[Tensor]<[1632]> weight = ?",
            "Optional[Tensor]<[1632]> bias = ?",
            "Tensor<[1632]> running_mean = ?",
            "Tensor<[1632]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 448, 12, 12]> input = ?",
            "Optional[Tensor]<[448]> weight = ?",
            "Optional[Tensor]<[448]> bias = ?",
            "Tensor<[448]> running_mean = ?",
            "Tensor<[448]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1280, 12, 12]> input = ?",
            "Optional[Tensor]<[1280]> weight = ?",
            "Optional[Tensor]<[1280]> bias = ?",
            "Tensor<[1280]> running_mean = ?",
            "Tensor<[1280]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 64, 150, 150]> input = ?",
            "Optional[Tensor]<[64]> weight = ?",
            "Optional[Tensor]<[64]> bias = ?",
            "Tensor<[64]> running_mean = ?",
            "Tensor<[64]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 150, 150]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 128, 75, 75]> input = ?",
            "Optional[Tensor]<[128]> weight = ?",
            "Optional[Tensor]<[128]> bias = ?",
            "Tensor<[128]> running_mean = ?",
            "Tensor<[128]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 75, 75]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 256, 38, 38]> input = ?",
            "Optional[Tensor]<[256]> weight = ?",
            "Optional[Tensor]<[256]> bias = ?",
            "Tensor<[256]> running_mean = ?",
            "Tensor<[256]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 728, 38, 38]> input = ?",
            "Optional[Tensor]<[728]> weight = ?",
            "Optional[Tensor]<[728]> bias = ?",
            "Tensor<[728]> running_mean = ?",
            "Tensor<[728]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 728, 19, 19]> input = ?",
            "Optional[Tensor]<[728]> weight = ?",
            "Optional[Tensor]<[728]> bias = ?",
            "Tensor<[728]> running_mean = ?",
            "Tensor<[728]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1024, 19, 19]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1024, 10, 10]> input = ?",
            "Optional[Tensor]<[1024]> weight = ?",
            "Optional[Tensor]<[1024]> bias = ?",
            "Tensor<[1024]> running_mean = ?",
            "Tensor<[1024]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 1536, 10, 10]> input = ?",
            "Optional[Tensor]<[1536]> weight = ?",
            "Optional[Tensor]<[1536]> bias = ?",
            "Tensor<[1536]> running_mean = ?",
            "Tensor<[1536]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
        [
            "Tensor<[1, 2048, 10, 10]> input = ?",
            "Optional[Tensor]<[2048]> weight = ?",
            "Optional[Tensor]<[2048]> bias = ?",
            "Tensor<[2048]> running_mean = ?",
            "Tensor<[2048]> running_var = ?",
            "float momentum = 0.1",
            "float eps = 0.001",
        ],
    ],
)
def test_aten(device, input_strings, input_var_only_native, input_var_check_accu, input_var_check_ttnn):
    metric = {
        "opname": "aten._native_batch_norm_legit_no_training.default",
        "input_strings": input_strings,
        "native_run": "N/A",
        "run": "N/A",
        "accuracy": "N/A",
        "convert_to_ttnn": "N/A",
        "ttnn_fallbacks_to_host_count": "N/A",
    }
    m = AtenModule()
    input_args, input_kwargs, status = render_metric_string_list_to_input_args_kwargs(
        "aten._native_batch_norm_legit_no_training.default", input_strings
    )
    if status == False:
        pytest.skip("Invalid input strings")
    try:
        result_before = m.forward(*input_args, **input_kwargs)
        metric["native_run"] = True
    except Exception as e:
        print(f"Failed to run native. Raised exception: {e}")
        metric["native_run"] = False

    if metric["native_run"] == True:
        result_after = None
        option = torch_ttnn.TorchTtnnOption(device=device)
        # option.gen_graphviz = True
        # The compilation is lazy, so we need to run forward once to trigger the compilation
        m = torch.compile(m, backend=torch_ttnn.backend, options=option)
        try:
            ttnn.graph.begin_graph_capture()
            result_after = m.forward(*input_args, **input_kwargs)
            # option._out_fx_graphs[0].print_tabular()
            metric["run"] = True
        except Exception as e:
            print(f"Failed to run. Raised exception: {e}")
            metric["run"] = False
        finally:
            trace = ttnn.graph.end_graph_capture()
            call_stack = ttnn.graph.extract_calltrace(trace)
            if metric["run"] == True:
                print(call_stack)
                expected_to_host_count = 0
                if result_after is None:
                    expected_to_host_count = 0
                elif isinstance(result_after, torch.Tensor):
                    expected_to_host_count = 1
                elif isinstance(result_after, (list, dict)):
                    expected_to_host_count = len(result_after)
                else:
                    print(f"Unexpected result_after type: {type(result_after)}")

                to_host_count = sum(["Tensor::cpu" in str(node) for node in call_stack])
                fallbacks_to_host_count = to_host_count - expected_to_host_count
                print(f"expected_to_host_count: {expected_to_host_count}")
                print(f"to_host_count: {to_host_count}")
                print(f"fallbacks_to_host_count: {fallbacks_to_host_count}")
                metric["ttnn_fallbacks_to_host_count"] = fallbacks_to_host_count

    if metric["run"] == True:
        try:
            # Check inference result
            metric["accuracy"] = calculate_accuracy(result_before, result_after)
        except Exception as e:
            print(f"Failed to check inference result. Raised exception: {e}")

        try:
            # Check the graph has be rewritten and contain ttnn ops
            nodes = list(option._out_fx_graphs[0].nodes)
            if not any(["aten." in str(node.target) for node in nodes]):
                metric["convert_to_ttnn"] = True
            else:
                metric["convert_to_ttnn"] = False
        except Exception as e:
            print(f"Failed to check the graph has ttnn op. Raised exception: {e}")

    metrics.append(metric)

    if not input_var_only_native:
        assert metric["run"] == True
        if input_var_check_accu:
            assert metric["accuracy"] >= 0.99
        if input_var_check_ttnn:
            assert metric["convert_to_ttnn"] == True
